#!/usr/bin/perl 

use lib '/usr/sausalito/perl';
use CCE;

my $cce = new CCE(Namespace => 'Time');
$cce->connectfd();

my $localtime = '/etc/localtime';
my $clock = '/etc/sysconfig/clock';
my $oid = $cce->event_oid();
my $time_obj = $cce->event_object();
my $old = $cce->event_old();

# We can set the time and defer application to the OS using the
# deferCommit boolean flag.  Time chanes will be applied by a 
# constructor on CCE restart
unless($time_obj->{deferCommit})
{
	# set the timezone first
	my $zone = $time_obj->{timeZone};

	# Obnoxious glibc UTC sign swap
	if ($zone =~ /GMT\+\d+/) {
		$zone =~ s/\+/\-/;
	} elsif ($zone =~ /GMT\-\d+/) {
		$zone =~ s/\-/\+/;
	}

	my $link = '../usr/share/zoneinfo/' . $zone;
	if ($zone and (readlink($localtime) ne $link)) {
		unlink('/etc/localtime');
		symlink($link, '/etc/localtime');
	}

	# update /etc/sysconfig/clock
	my $fn = sub {
		my ($fin, $fout) = (shift,shift);
		my ($text) = (shift);

		while (<$fin>) {
			if(m/^ZONE/) {
				# print out the CCE maintained section
				print $fout "ZONE=\"$text\"\n";
			} else {
				print $fout $_;
			}
		}

		return 1;
	};

	if (!Sauce::Util::editfile($clock, $fn, $zone)) {
		$cce->warn("[[base-time.errorWritingConfFile]]");
	}

	# set the time if necessary. 
	my $time = $time_obj->{epochTime};

	if($old->{epochTime} && $time_obj->{epochOffset})
	{
		$time = $time + (time() - $time_obj->{epochOffset});
		$cce->set($oid, 'Time', {'epochOffset' => 0});
	}

	if (($time ne $old->{epochTime}) ||
	    ($old->{deferCommit})) 
	{
		
	    `/usr/sausalito/sbin/epochdate $time`;
	    # resync the hwclock with the time
	    system ("/sbin/hwclock --utc --systohc > /dev/null");
 	   unlink("/etc/adjtime"); # get rid of any clock skew
	    system ("/sbin/hwclock --utc --systohc > /dev/null");
	}

	# reload ntpd, if it's running, after the hw clock set
	Sauce::Service::service_run_init('ntpd', 'reload') if 
		($time_obj->{ntpAddress});

    # Restart Rsyslog:
    Sauce::Service::service_run_init('rsyslog', 'restart');

}

$cce->bye("SUCCESS");
exit 0;

# 
# Copyright (c) 2018-2020 Michael Stauber, SOLARSPEED.NET
# Copyright (c) 2018-2020 Team BlueOnyx, BLUEONYX.IT
# Copyright (c) 2003 Sun Microsystems, Inc. 
# All Rights Reserved.
# 
# 1. Redistributions of source code must retain the above copyright 
#     notice, this list of conditions and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright 
#     notice, this list of conditions and the following disclaimer in 
#     the documentation and/or other materials provided with the 
#     distribution.
# 
# 3. Neither the name of the copyright holder nor the names of its 
#     contributors may be used to endorse or promote products derived 
#     from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
# POSSIBILITY OF SUCH DAMAGE.
# 
# You acknowledge that this software is not designed or intended for 
# use in the design, construction, operation or maintenance of any 
# nuclear facility.
# 